@@ -11,7 +11,6 @@ import logging
 import os
 import warnings
 
-from jinja2 import Template
 from fsspec import open_files, get_filesystem_class
 from fsspec.core import split_protocol
 
@@ -80,7 +79,7 @@ class UserParameter(DictSerialiseMixin):
                             for item in self.allowed]
 
     def __repr__(self):
-        return (f'<{self.__class__.__name__} {self.name!r}>')
+        return f'<{self.__class__.__name__} {self.name!r}>'
 
     __str__ = __repr__
 
@@ -166,6 +165,7 @@ class LocalCatalogEntry(CatalogEntry):
             Catalog object in which this entry belongs
         """
         self._name = name
+        self._default_source = None
         self._description = description
         self._driver = driver
         self._direct_access = direct_access
@@ -225,7 +225,8 @@ class LocalCatalogEntry(CatalogEntry):
         return {
             'name': self._name,
             'container': self._container,
-            'plugin': pl,
+            'plugin': pl,   # deprecated
+            'driver': pl,
             'description': self._description,
             'direct_access': self._direct_access,
             'user_parameters': [u.describe() for u in self._user_parameters],
@@ -277,6 +278,9 @@ class LocalCatalogEntry(CatalogEntry):
 
     def get(self, **user_parameters):
         """Instantiate the DataSource for the given parameters"""
+        if not user_parameters and self._default_source is not None:
+            return self._default_source
+
         plugin, open_args = self._create_open_args(user_parameters)
         data_source = plugin(**open_args)
         data_source.catalog_object = self._catalog
@@ -284,8 +288,19 @@ class LocalCatalogEntry(CatalogEntry):
         data_source.description = self._description
         data_source.cat = self._catalog
 
+        # Cache the default source if there are no user parameters.
+        if not user_parameters:
+            self._default_source = data_source
+
         return data_source
 
+    def clear_cached_default_source(self):
+        """
+        Clear a cached default source so it can be created anew (if, for instance,
+        it depends on changing environment variables or execution context)
+        """
+        self._default_source = None
+
 
 class CatalogParser(object):
     """Loads entries from a YAML spec"""
@@ -549,6 +564,7 @@ class YAMLFileCatalog(Catalog):
         self.text = None
         self.autoreload = autoreload  # set this to False if don't want reloads
         self.filesystem = kwargs.pop('fs', None)
+        self.access = "name" not in kwargs
         super(YAMLFileCatalog, self).__init__(**kwargs)
 
     def _load(self, reload=False):
@@ -557,6 +573,11 @@ class YAMLFileCatalog(Catalog):
         Will do nothing if auto-reload is off and reload is not explicitly
         requested
         """
+        if self.access is False:
+            # skip first load, if cat has given name (i.e., is subcat)
+            self.updated = 0
+            self.access = True
+            return
         if self.autoreload or reload:
             # First, we load from YAML, failing if syntax errors are found
             options = self.storage_options or {}
@@ -707,11 +728,17 @@ class YAMLFilesCatalog(Catalog):
         self._kwargs = kwargs.copy()
         self._cat_files = []
         self._cats = {}
+        self.access = "name" not in kwargs
         super(YAMLFilesCatalog, self).__init__(**kwargs)
 
     def _load(self):
         # initial: find cat files
         # if flattening, need to get all entries from each.
+        if self.access is False:
+            # skip first load, if cat has given name (i.e., is subcat)
+            self.updated = 0
+            self.access = True
+            return
         self._entries.clear()
         options = self.storage_options or {}
         if isinstance(self.path, (list, tuple)):
@@ -752,18 +779,20 @@ class YAMLFilesCatalog(Catalog):
                 else:
                     # store a catalog entry
                     self._cats[f.path] = entry
+        entries = {}
         for name, entry in list(self._cats.items()):
             if self._flatten:
                 entry.reload()
-                inter = set(entry._entries).intersection(self._entries)
+                inter = set(entry._entries).intersection(entries)
                 if inter:
                     raise ValueError(
                         'Conflicting names when flattening multiple'
                         ' catalogs. Sources %s exist in more than'
                         ' one' % inter)
-                self._entries.update(entry._entries)
+                entries.update(entry._entries)
             else:
-                self._entries[entry._name] = entry
+                entries[entry._name] = entry
+        self._entries = entries
 
 
 class MergedCatalog(Catalog):
@@ -777,9 +806,9 @@ class MergedCatalog(Catalog):
     def _load(self):
         for catalog in self._catalogs:
             catalog._load()
-
-    def _make_entries_container(self):
-        return collections.ChainMap(*(catalog._entries for catalog in self._catalogs))
+        self._entries = collections.ChainMap(
+            *(catalog._entries for catalog in self._catalogs)
+        )
 
 
 class EntrypointEntry(CatalogEntry):
@@ -787,14 +816,15 @@ class EntrypointEntry(CatalogEntry):
     A catalog entry for an entrypoint.
 
     """
-    _container = 'catalog'
 
     def __init__(self, entrypoint):
         self._entrypoint = entrypoint
+        self._container = None
+        self._user_parameters = []
         super().__init__()
 
     def __repr__(self):
-        return f"<Entry containing Catalog named {self.name}>"
+        return f"<Entry '{self.name}'>"
 
     @property
     def name(self):
@@ -802,12 +832,14 @@ class EntrypointEntry(CatalogEntry):
 
     def describe(self):
         """Basic information about this entry"""
+        if self._container is None:
+            self._container = self().container
         return {'name': self.name,
                 'module_name': self._entrypoint.module_name,
                 'object_name': self._entrypoint.object_name,
                 'distro': self._entrypoint.distro,
                 'extras': self._entrypoint.extras,
-                'container': ""
+                'container': self._container
                 }
 
     def get(self):
